<?php
if (!function_exists('curl_init')) die ('You must have curl installed to use the supercontainer php library');

/**
* This class is a PHP API for interation with <a href="http://360works.com/supercontainer">SuperContainer</a>.
* This provides an easy-to-use object-oriented API for uploading, downloading, and deleting files from a SuperContainer server, 
* in addition to getting metadata about files on a SuperContainer server.  
* This is especially useful for custom web publishing (CWP) FileMaker solutions using PHP.
* <p>
* <h3>Requirements</h3>
* This requires that PHP be built with <a href="http://www.php.net/manual/ref.curl.php">curl library support</a> enabled.
* <p>
* There are no other dependencies.
* <h3>Example Usage</h3>
* To use the SuperContainer lib, you first create a SuperContainer object:
* <pre class="php">
* $sc = new SuperContainer('http://server.mycompany.com:8020/SuperContainer/Files');
* </pre>
* Then you can use this object to upload a file:
* <pre class="php">
* $sc->upload('uploads/12345', '/path/to/my/local-file.txt');
* </pre>
* Get metadata about an uploaded file:
* <pre class="php">
* $infoArray = $sc->getInfo('uploads/12345');
* $fileName = $infoArray['name'];
* $fileSize = $infoArray['size'];
* $uploadDate = $infoArray['date'];
* </pre>
* Download the SuperContainer file to your local drive:
* <pre class="php">
* $localPath = $sc->download('uploads/12345', '/tmp');
* </pre>
* Or delete an uploaded file:
* <pre class="php">
* $sc->delete('uploads/12345');
* </pre>
* <p>
* To handle file uploads in PHP, move the uploaded file using the <code>move_uploaded_file</code> function, then the <code>upload</code> function of SuperContainer.
* The following example illustrates handling a file upload, and better error handling.
* <br style="page-break-after:always;"/>
* <pre class="php">
* $SC_BASE_URL = 'http://server.mycompany.com:8020/SuperContainer/Files';
* $sc = new SuperContainer($SC_BASE_URL);
* if ($sc->getError()) die('Could not connect to SuperContainer server: ' . $sc->getError());
* $uploadedFile = sys_get_temp_dir() . '/' . $_FILES['upload']['name'];
* move_uploaded_file($_FILES['upload']['tmp_name'], $uploadedFile) 
*     or die("Could not move uploaded file to $uploadedFile");
* $folderPath = 'uploads/12345';
* $sc->upload($folderPath, $uploadedFile) 
*     or die("Could not upload $uploadedFile to $folderPath: " . $sc->getError());
* </pre>
*/
class SuperContainer {
	var $baseUrl, $urlParts, $error;
	var $version = '2.864';

	/**
	* Constructor which accepts a baseUrl, optional username, optional password.<p>
	* Note that values cannot be returned from a constructor, therefore it is difficult to validate the return value.  
	* To verify that the <code>SuperContainer</code> object was created correctly, you should call <code>getError()</code> immediately after creating the new object.
	* @param string $baseUrl The Base URL of your SuperContainer server, e.g. <code>http://server.mycompany.com:8020/SuperContainer/Files</code>
	* @param string $user Optional username, only required if your SuperContainer server is password protected
	* @param string $password Optional password, only required if your SuperContainer server is password protected
	*/
	function SuperContainer($baseUrl, $user='', $password='') {
		if (!$this->_validateRequired($baseUrl, 'baseUrl')) {
			return false;
		}
		$this->error = null;
		$this->baseUrl = $baseUrl;
		$this->urlParts = parse_url($baseUrl);
		if ($user) $this->urlParts['user'] = $user;
		if ($password) $this->urlParts['password'] = $password;
		// validate that the server is running
		$ch = $this->_createCurl($this->baseUrl);
		curl_exec($ch);
		$this->_cleanup($ch); // this will set an error if something went wrong
	}

	/**
	* Returns the version number of the PHP api.
	*/
	function getVersion() {
		return $this->version;
	}
	/**
	* Gets the last SuperContainer error message.  Use this to log errors or display additional information to users.
	* @returns string
	*/
	function getError() {
		return $this->error;
	}

	/**
	* Upload a file to the SuperContainer server.  Returns <code>true</code> if the upload was successful, false if not.
	* @param string $folderPath The unique identifier for a supercontainer file.
	* @param string $data local path to the file to upload.
	* @returns boolean
	*/
	function upload($folderPath, $data) {
		// validate input
		if (!$this->_validateRequired($folderPath, 'folderPath')) return false;
		if (!$this->_validateRequired($data, 'data')) return false;
		if (!file_exists($data)) {
			$this->error = "File does not exist: '$data'";
			return false;
		}
		// end validation
		$this->error = null;
		$host = $this->urlParts['host'];
		$port = $this->urlParts['port'];

		$url = $this->baseUrl . '/' . $folderPath;
		$ch = $this->_createCurl($url);
		curl_setopt($ch, CURLOPT_POST, 1);
		$postFields = array('file'=>'@' . realpath($data));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $postFields);
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		// curl_setopt($ch, CURLOPT_UPLOAD, true); // prepare for an upload.  Uncommenting this breaks the request, returns a 405 error
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10); // The number of seconds to wait whilst trying to connect. Use 0 to wait indefinitely
		curl_exec($ch);
		$this->_cleanup($ch);
		return empty($this->error);
	}

	/**
	* Download a file from the SuperContainer server to the local filesystem.
	* This returns the final path of the downloaded file, or <code>false</code> if the download failed, or <code>''</code> if there was no file to download at the specified <code>folderPath</code>.
	* <p>
	* <strong>Note:</strong> If <code>$destination</code> is a directory, the downloaded file will be given the same name it had on the SuperContainer server.  If <code>$destination</code> is not a directory, the file will be downloaded to <code>$destination</code> without altering the path at all.  Note that the parent directory must exist for this operation to complete successfully.
	* @param string $folderPath The unique identifier for the supercontainer file to download
	* @param string $destination The local destination path to download the file to
	* @returns string
	*/
	function download($folderPath, $destination) {
		if (!$this->_validateRequired($folderPath, 'folderPath')) return false;
		if (!file_exists(dirname($destination))) {
			$this->error = "Download directory does not exist, or is not writeable: '$destination'";
			return false;
		}
		// end validation
		// first check for whether there is a file
		$info = $this->getInfo($folderPath);
		if ($info == array()) return ''; // no file to download
		if (!$info) return false; // something went wrong when getting info
		$this->error = null;
		$ch = $this->_createCurl(dirname($this->baseUrl) . '/RawData/' . $folderPath);
		$tmpPath = tempnam('/tmp', 'supercontainer-download-');
		$fp = fopen($tmpPath, 'w+');
		curl_setopt($ch, CURLOPT_FILE, $fp);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_exec($ch);
		fclose($fp);
		// now move the tmp file to the real destination
		if (is_dir($destination)) {
			$destination .= '/' . $info['name'];
		}
		rename($tmpPath, $destination);
		// cleanup and return
		$this->_cleanup($ch);
		return $destination;
	}

	/**
	* Retrieve information about a SuperContainer file without actually download it. This returns an associative array with the following keys:
	* <dl>
	* <dt>name</dt><dd>The name of the file</dd>
	* <dt>size</dt><dd>The size in bytes of the file</dd>
	* <dt>date</dt><dd>The date the file was uploaded to SuperContainer (use <code>strtotime()</code> to convert this to a timestamp)</dd>
	* </dl>
	* <p>
	* If there is no file at the given <code>folderPath</code>, an empty array is returned. However, calling <code>getError</code> will return an empty string.
	* @param string $folderPath The unique identifier for the supercontainer file to download
	* @returns array
	*/
	function getInfo($folderPath) {
		if (!$this->_validateRequired($folderPath, 'folderPath')) return false;
		$this->error = null;
		$url = $this->baseUrl . '/' . $folderPath;
		$ch = $this->_createCurl($url);
		// curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'HEAD'); // this causes things to hang forever, it seems
		curl_setopt($ch, CURLOPT_HEADER, true); // include the header in the output
		curl_setopt($ch, CURLOPT_NOBODY, true); // exclude the body from the output
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false); // don't redirect, we want the headers from SuperContainer
		$response = curl_exec($ch); // this should just contain the headers
		// filter out some headers
		$matches = array();
		$result = array();
		if (preg_match('/X-SuperContainer-Filename:\\s*(.*)/', $response, $matches)) {
			$result['name'] = $matches[1];
			preg_match('/X-SuperContainer-Filesize:\\s*(.*)/', $response, $matches);
			$result['size'] = $matches[1];
			preg_match('/Last-Modified:\\s*(.*)/', $response, $matches);
			$result['date'] = $matches[1];
		} else {
			// no uploaded file on SuperContainer
			// we'll return an empty array, but error will be empty
		}

		$this->_cleanup($ch);
		return $result; // may return an empty array
	}

	/**
	* Delete a file from the SuperContainer server.
	* Returns <code>true</code> if the delete operation was successful, or there was no file to delete.  Otherwise, returns <code>false</code>.
	* @param string $folderPath The unique identifier for the supercontainer file to download
	* @returns boolean
	*/
	function delete($folderPath) {
		if (!$this->_validateRequired($folderPath, 'folderPath')) return false;
		$this->error = null;
		$ch = $this->_createCurl($this->baseUrl . '/' . $folderPath);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, 'delete=Delete');
		curl_exec($ch);
		$this->_cleanup($ch);
		return empty($this->error);
	}

	private function _createCurl($url) {
		$ch = curl_init($url);
		if (!empty($this->urlParts['password'])) {
			curl_setopt($ch, CURLOPT_USERPWD, $this->urlParts['user'] . ':' . $this->urlParts['password']);// replace baseURL with urlParts
		}
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_USERAGENT, "SuperContainer PHP API");
		curl_setopt($ch, CURLOPT_FAILONERROR, true); // fail silently if the HTTP code returned is greater than or equal to 400.
		return $ch;
	}

	private function _cleanup($ch) {
		if (!$this->error) $this->error = curl_error($ch);
		curl_close($ch);
	}

	private function _validateRequired($value, $name) {
		if (empty($value)) {
			$this->error = 'No ' . $name . ' specified';
			return false;
		}
		return true;
	}

}

?>
